@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
-# Gettings Things Gnome! - a personnal organizer for the GNOME desktop
+# Gettings Things Gnome! - a personal organizer for the GNOME desktop
 # Copyright (c) 2008-2009 - Lionel Dricot & Bertrand Rousseau
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License along with
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#===============================================================================
+#==============================================================================
 #
 # Getting things Gnome!: a gtd-inspired organizer for GNOME
 #
@@ -38,60 +38,80 @@
 #   Each id are *strings*
 #   tid are the form "X@Y" where Y is the pid.
 #   For example : 21@2 is the 21th task of the 2nd project
-#   This way, we are sure that a tid is unique accross multiple projects 
+#   This way, we are sure that a tid is unique accross multiple projects
 #
-#=============================================================================== 
+#==============================================================================
 
-#=== IMPORT ====================================================================
-import sys, os
+#=== IMPORT ===================================================================
+from __future__ import with_statement
+
+import sys
+import os
+import dbus
+import logging
+import signal
+from contextlib import contextmanager
 
 #our own imports
+from GTG import _
 from GTG.taskbrowser.browser import TaskBrowser
-from GTG.core.datastore      import DataStore
-from GTG.core                import CoreConfig
+from GTG.core.datastore import DataStore
+from GTG.core.dbuswrapper import DBusTaskWrapper
+from GTG.core import CoreConfig
 
-#=== OBJECTS ===================================================================
+#=== OBJECTS ==================================================================
 
 #code borrowed from Specto. Avoid having multiples instances of gtg
 #reading the same tasks
 #that's why we put the pid file in the data directory :
 #we allow one instance of gtg by data directory.
+
 def check_instance(directory):
-    """ Check if gtg is already running. """
-    pidfile = directory  + "gtg.pid"
+    """Check if gtg is already running."""
+    pidfile = os.path.join(directory, "gtg.pid")
     if not os.path.exists(pidfile):
-        f = open(pidfile, "w")
-        f.close()
-    os.chmod(pidfile, 0600)
+        open(pidfile, "w").close()
+        os.chmod(pidfile, 0600)
 
     #see if gtg is already running
-    f = open(pidfile, "r")
-    pid = f.readline()
-    f.close()
+    pid = open(pidfile, "r").readline()
     if pid:
-        p = os.system("ps --no-heading --pid " + pid)
-        p_name = os.popen("ps -f --pid " + pid).read()
+        p = os.system("/bin/ps %s >/dev/null" % pid)
+        p_name = os.popen("/bin/ps -f %s" % pid).read()
         if p == 0 and "gtg" in p_name:
             print _("gtg is already running!")
-            #todo : expose the browser (will be possible when we have dbus)
+            d=dbus.SessionBus().get_object(CoreConfig.BUSNAME,\
+                                           CoreConfig.BUSINTERFACE)
+            d.show_task_browser()
             sys.exit(0)
             
     #write the pid file
-    f = open(pidfile, "w")
-    f.write(str(os.getpid()))
-    f.close()
+    with open(pidfile, "w") as f:
+        f.write(`os.getpid()`)
 
-#=== MAIN CLASS ================================================================
+#=== MAIN CLASS ===============================================================
 
-def main():
-    config        = CoreConfig()
+def main(options=None, args=None):
+    
+    # init logging system
+    logger = logging.getLogger("gtg_logger")
+    ch = logging.StreamHandler()
+    ch.setLevel(logging.DEBUG)
+    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(module)s:%(funcName)s:%(lineno)d - %(message)s")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+    if options.debug:
+        logger.setLevel(logging.DEBUG)
+        logger.debug("Debug output enabled.")
+    
+    config = CoreConfig()
     check_instance(config.DATA_DIR)
     backends_list = config.get_backends_list()
     
     # Load data store
     ds = DataStore()
     
-    for backend_dic in backends_list :
+    for backend_dic in backends_list:
         ds.register_backend(backend_dic)
     
     #save directly the backends to be sure to write projects.xml
@@ -99,8 +119,20 @@ def main():
         
     # Launch task browser
     req = ds.get_requester()
-    tb  = TaskBrowser(req, config.conf_dict)
-    tb.main()
+    tb = TaskBrowser(req, config, logger=logger)
+    DBusTaskWrapper(req, tb)
+    #we listen for signals from the system in order to save our configuration
+    # if GTG is forcefully terminated (e.g.: on shutdown).
+    @contextmanager
+    def signal_catcher():
+        #if TERM or ABORTare caught, we close the browser
+        for s in [signal.SIGABRT, signal.SIGTERM]:
+            signal.signal(s, lambda a,b: tb.on_close())
+        yield
+
+    #main loop
+    with signal_catcher():
+        tb.main()
 
     # Ideally we should load window geometry configuration from a config.
     # backend like gconf at some point, and restore the appearance of the
@@ -110,7 +142,7 @@ def main():
     config.save_config()
     config.save_datastore(ds)
 
-#=== EXECUTION =================================================================
+#=== EXECUTION ================================================================
 
 if __name__ == "__main__":
     main()
